@@ -1,7 +1,6 @@
-/* $Id: bswap.c 2351 2006-04-05 15:08:34Z  $ */
-#include "synce.h"
+/* $Id: bswap.c 3686 2009-01-29 12:58:25Z mark_ellis $ */
 
-#if IMPLEMENT_BSWAP_XX
+#include "synce.h"
 
 /*
  * Written by Manuel Bouyer <bouyer@netbsd.org>.
@@ -21,5 +20,3 @@ uint32_t bswap_32(uint32_t x)
 		((x >> 24) & 0x000000ff );
 }
 
-#endif
-
